import os
from os import rename
from os.path import join
from typing import Tuple

import datatable as dt
from datatable import Frame

from rs_datasets.data_loader import download_dataset
from rs_datasets.generic_dataset import Dataset, safe

rating_cols = ['user_id', 'item_id', 'rating', 'timestamp']
item_cols = ['item_id', 'title', 'genres']
tag_cols = ['user_id', 'item_id', 'tag', 'timestamp']
user_cols = ['user_id', 'gender', 'age', 'occupation', 'zip_code']
link_cols = ['item_id', 'imdb_id', 'tmdb_id']
tag_g_cols = ['tag_id', 'tag']
score_cols = ['movie_id', 'tag_id', 'rating']
genre_cols = ['item_id', 'title', 'release_date', 'video_release_date',
              'imdb_url', 'unknown', 'Action', 'Adventure', 'Animation',
              'Children\'s', 'Comedy', 'Crime', 'Documentary', 'Drama',
              'Fantasy', 'Film-Noir', 'Horror', 'Musical', 'Mystery',
              'Romance', 'Sci-Fi', 'Thriller', 'War', 'Western']

class MovieLens(Dataset):
    def __init__(
            self,
            version: str = 'small',
            read_genome: bool = False,
            path: str = None
    ):
        """
        :param version: dataset version,
            one of {'100k', '1m', '10m', '20m', '25m', 'small', 'latest'}
        :param read_genome: whether to read genome tag dataset or not
            (available from version 20m and up).
            Are not loaded by default to save memory.
        :param path: where to read dataset from or where to download to.
        """
        super().__init__(path)
        options = {'100k', '1m', '10m', '20m', '25m', 'small', 'latest'}
        if version not in options:
            raise ValueError(
                f'{version} is not supported. Available options: {options}')

        if version == 'small':
            dataset = 'ml-latest-small'
        else:
            dataset = 'ml-' + version

        folder = join(self.data_folder, dataset)
        if not os.path.exists(folder):
            self._download(folder, dataset)
        
        if version == '100k':
            (self.ratings,
             self.users,
             self.items) = self._read_100k(folder)
        elif version == '1m':
            (self.ratings,
             self.users,
             self.items) = self._read_1m(folder)
        elif version == '10m':
            (self.ratings,
             self.items,
             self.tags) = self._read_10m(folder)
        else:
            (self.ratings,
             self.items,
             self.tags,
             self.links) = self._read_modern(folder)
            if read_genome:
                (self.genome_tags,
                 self.genome_scores) = self._read_genome(folder)

    @staticmethod
    def _read_modern(folder: str) -> Tuple[Frame, Frame, Frame, Frame]:
        ratings = dt.fread(join(folder, 'ratings.csv'), columns=rating_cols).to_pandas()
        items = dt.fread(join(folder, 'movies.csv'), columns=item_cols).to_pandas()
        tags = dt.fread(join(folder, 'tags.csv'), columns=tag_cols).to_pandas()
        links = dt.fread(join(folder, 'links.csv'), columns=link_cols).to_pandas()
        return ratings, items, tags, links

    @staticmethod
    def _read_genome(folder: str) -> Tuple[Frame, Frame]:
        genome_tags = dt.fread(join(folder, 'genome-tags.csv'), columns=tag_g_cols).to_pandas()
        genome_scores = dt.fread(join(folder, 'genome-scores.csv'), columns=score_cols).to_pandas()
        return genome_tags, genome_scores

    @staticmethod
    def _read_10m(folder: str) -> Tuple[Frame, Frame, Frame]:
        ratings = dt.fread(join(folder, 'ratings.dat'), columns=rating_cols).to_pandas()
        items = dt.fread(join(folder, 'movies.dat'), columns=item_cols, quotechar="").to_pandas()
        tags = dt.fread(join(folder, 'tags.dat'), columns=tag_cols, quotechar="").to_pandas()
        return ratings, items, tags

    @staticmethod
    def _read_1m(folder: str) -> Tuple[Frame, Frame, Frame]:
        ratings = dt.fread(join(folder, 'ratings.dat'), columns=rating_cols).to_pandas()
        users = dt.fread(join(folder, 'users.dat'), columns=user_cols).to_pandas()
        items = dt.fread(join(folder, 'movies.dat'), columns=item_cols).to_pandas()
        return ratings, users, items

    @staticmethod
    def _read_100k(folder: str) -> Tuple[Frame, Frame, Frame]:
        ratings = dt.fread(join(folder, 'u.data'), columns=rating_cols).to_pandas()
        users = dt.fread(join(folder, 'u.user'), columns=user_cols).to_pandas()
        items = dt.fread(join(folder, 'u.item'), columns=genre_cols)
        del items[:, 'video_release_date']
        items = items.to_pandas()
        return ratings, users, items

    @safe
    def _download(self, path, dataset):
        """
        Download data from https://grouplens.org/datasets/movielens/
        Available options: ml-20m, ml-latest-small, ml-latest and other,
        can be checked on ml site.

        :param path: where to save
        :param dataset: dataset version
        :return: None
        """
        self.logger.info('Downloading %s from grouplens...', dataset)
        archive = dataset + '.zip'
        url = f'http://files.grouplens.org/datasets/movielens/{archive}'
        download_dataset(url, path + '.zip')
        if dataset == 'ml-10m':
            rename(join(self.data_folder, 'ml-10M100K'), path)
            self.replace_separator(join(path, 'movies.dat'), '::', '\t')
            self.replace_separator(join(path, 'ratings.dat'), '::', '\t')
            self.replace_separator(join(path, 'tags.dat'), '::', '\t')
        elif dataset == 'ml-1m':
            self.replace_separator(join(path, 'movies.dat'), '::', '\t', 'ISO-8859-1')
            self.replace_separator(join(path, 'ratings.dat'), '::', '\t')
            self.replace_separator(join(path, 'users.dat'), '::', '\t')

    @staticmethod
    def replace_separator(filepath: str, old: str, new: str, encoding: str = 'utf8'):
        with open(filepath, 'r', encoding=encoding) as f:
            newlines = []
            for line in f.readlines():
                newlines.append(line.replace(old, new))
        with open(filepath, 'w') as f:
            for line in newlines:
                f.write(line)



import logging
from os.path import join, exists

import datatable as dt

from rs_datasets.generic_dataset import Dataset, safe


class Anime(Dataset):
    def __init__(self, path: str = None):
        super().__init__(path)
        folder = join(self.data_folder, 'anime')
        if not exists(folder):
            self._download(folder)

        self.ratings = dt.fread(
            join(folder, 'rating.csv'),
            columns=[
                'user_id', 'item_id', 'rating'
            ],
        ).to_pandas()

        self.titles = dt.fread(
            join(folder, 'anime.csv'),
            columns=[
                'item_id', 'name', 'genre', 'type', 'episodes', 'rating', 'members'
            ],
        ).to_pandas()

    @safe
    def _download(self, path):
        from kaggle.api.kaggle_api_extended import KaggleApi
        logging.info('Downloading anime dataset...')
        api = KaggleApi()
        api.authenticate()
        api.dataset_download_files('CooperUnion/anime-recommendations-database', path, unzip=True)



import json
import logging
from os import rename
from os.path import join, exists

import datatable as dt

from rs_datasets.data_loader import download_dataset
from rs_datasets.generic_dataset import Dataset, safe


class Rekko(Dataset):
    def __init__(self, path: str = None):
        super().__init__(path)
        folder = join(self.data_folder, 'rekko')
        if not exists(folder):
            self._download(folder)

        self.transactions = dt.fread(
            join(folder, 'transactions.csv'),
            columns=[
                'item_id', 'user_id', 'consumption_mode', 'ts',
                'watched_time', 'device_type', 'device_manufacturer'
            ],
        ).to_pandas()

        self.ratings = dt.fread(
            join(folder, 'ratings.csv'),
            columns=['user_id', 'item_id', 'rating', 'ts']
        ).to_pandas()

        self.bookmarks = dt.fread(
            join(folder, 'bookmarks.csv'),
            columns=['user_id', 'item_id', 'ts']
        ).to_pandas()

        with open(join(folder, 'catalogue.json')) as json_file:
            self.catalogue = json.load(json_file)


    @safe
    def _download(self, path):
        logging.info('Downloading rekko challenge dataset...')
        archive = 'rekko_challenge_rekko_challenge_2019.zip'
        url = f'https://boosters.pro/api/ch/files/pub/{archive}'
        download_dataset(url, join(self.data_folder, 'rekko.zip'))
        rename(join(self.data_folder, 'rekko'), path)



import os
from os.path import join, exists

import datatable as dt
import gdown

from rs_datasets.data_loader import extract
from rs_datasets.generic_dataset import Dataset, safe


class Diginetica(Dataset):
    def __init__(self, path: str = None):
        """
        :param path: folder which is used to download dataset to
            if it does not contain dataset files.
            If files are found, load them.
        """
        super().__init__(path)
        folder = join(self.data_folder, 'diginetica')
        if not exists(folder):
            self._download(folder)

        self.items = dt.fread(
            join(folder, 'products.csv'),
            columns=['item_id', 'log2price', 'name_tokens']
        ).to_pandas()

        self.categories = dt.fread(
            join(folder, 'product-categories.csv'),
            columns=['item_id', 'category_id']
        ).to_pandas()

        self.purchases = dt.fread(
            join(folder, 'train-purchases.csv'),
            columns=['session_id', 'user_id', 'timeframe', 'date', 'order_id', 'item_id']
        ).to_pandas()

        self.views = dt.fread(
            join(folder, 'train-item-views.csv'),
            columns=['session_id', 'user_id', 'item_id', 'timeframe', 'date']
        ).to_pandas()

        self.queries = dt.fread(
            join(folder, 'train-queries.csv'),
            columns=['query_id', 'session_id', 'user_id', 'timeframe', 'duration', 'date',
                     'tokens', 'category_id', 'items', 'is_test']
        ).to_pandas()



    @safe
    def _download(self, path):
        self.logger.info('Downloading Diginetica Dataset...')
        url = 'https://drive.google.com/uc?id=0B7XZSACQf0KdenRmMk8yVUU5LWc'
        zip = join(self.data_folder, 'diginetica.zip')
        gdown.download(url, zip)
        extract(zip)
        os.remove(zip)



import logging
from os.path import join, exists

import datatable as dt

from rs_datasets.generic_dataset import Dataset, safe


class Steam(Dataset):
    def __init__(self, path: str = None):
        super().__init__(path)
        folder = join(self.data_folder, 'steam')
        if not exists(folder):
            self._download(folder)

        self.data = dt.fread(
            join(folder, 'steam-200k.csv'),
            columns=[
                'user_id', 'game', 'behavior', 'value', 'c'
            ],
        ).to_pandas()
        self.data = self.data.drop('c', axis=1)

    @safe
    def _download(self, path):
        from kaggle.api.kaggle_api_extended import KaggleApi
        logging.info('Downloading steam dataset...')
        api = KaggleApi()
        api.authenticate()
        api.dataset_download_files('tamber/steam-video-games', path, unzip=True)



from rs_datasets.movielens import MovieLens
from rs_datasets.msd import MillionSongDataset
from rs_datasets.netflix import Netflix
from rs_datasets.lastfm import Lastfm
from rs_datasets.epinions import Epinions
from rs_datasets.goodreads import Goodreads
from rs_datasets.book_crossing import BookCrossing
from rs_datasets.jester import Jester
from rs_datasets.amazon import Amazon
from rs_datasets.rekko import Rekko
from rs_datasets.steam import Steam
from rs_datasets.anime import Anime
from rs_datasets.retail_rocket import RetailRocket
from rs_datasets.yoochoose import YooChoose
from rs_datasets.diginetica import Diginetica



from os import mkdir
from os.path import join, exists

import datatable as dt
import gdown

from rs_datasets.generic_dataset import Dataset, safe


class Goodreads(Dataset):
    def __init__(self, path: str = None, read_maps: bool = False):
        """
        :param path: folder which is used to download dataset to
            if it does not contain dataset files.
            If files are found, load them.
        :param read_maps: ids in interactions are encoded to save memory.
            You can read mappings, but there's no point
            if you don't use the rest of the dataset, which is not included here.
        """
        super().__init__(path)
        folder = join(self.data_folder, 'goodreads')
        if not exists(folder):
            self._download(folder)

        self.ratings = dt.fread(
            join(folder, 'goodreads_interactions.csv'),
            columns=['user_id', 'item_id', 'is_read', 'rating', 'is_reviewed']
        ).to_pandas()

        if read_maps:
            self.books = dt.fread(
                join(folder, 'book_id_map.csv')
            ).to_pandas()

            self.users = dt.fread(
                join(folder, 'user_id_map.csv')
            ).to_pandas()

    @safe
    def _download(self, path):
        """
        https://sites.google.com/eng.ucsd.edu/ucsdbookgraph/shelves
        https://github.com/MengtingWan/goodreads/blob/master/download.ipynb
        """
        self.logger.info('Downloading interactions from Goodreads dataset...')
        mkdir(path)

        gdrive = 'https://drive.google.com/uc?id='
        interactions = gdrive + '1zmylV7XW2dfQVCLeg1LbllfQtHD2KUon'
        users = gdrive + '15ax-h0Oi_Oyee8gY_aAQN6odoijmiz6Q'
        items = gdrive + '1CHTAaNwyzvbi1TR08MJrJ03BxA266Yxr'

        gdown.download(users, join(path, 'user_id_map.csv'))
        gdown.download(items, join(path, 'book_id_map.csv'))
        gdown.download(interactions, join(path, 'goodreads_interactions.csv'))



from glob import glob
from os import mkdir, remove, rename
from os.path import exists, join

import datatable as dt
import pandas as pd
from tqdm import tqdm

from rs_datasets.data_loader import download_dataset, extract, rm_if_exists
from rs_datasets.generic_dataset import Dataset, safe


class Netflix(Dataset):
    def __init__(self, path: str = None):
        """
        :param path: where to read dataset from or where to download to.
        """
        super().__init__(path)
        folder = join(self.data_folder, 'netflix')
        if not exists(folder):
            self._download(folder)
            self._save_clean(folder)
        self._read_clean(folder)

    def _read_clean(self, folder):
        path = join(folder, 'clean')
        self.movies = pd.read_csv(join(path, 'movies.csv'), sep='\t',
                                  names=['item_id', 'year', 'title'],
                                  dtype={'item_id': 'uint16',
                                         'year': 'float32'})
        test = dt.fread(
            join(path, 'test.csv'),
            columns=['item_id', 'user_id', 'timestamp']
        ).to_pandas()
        test['timestamp'] = pd.to_datetime(test['timestamp'])
        test['user_id'] = test['user_id'].astype('category')
        test['item_id'] = test['item_id'].astype('category')
        self.test = test

        if exists(join(path, 'train.parquet')):
            self.train = pd.read_parquet(join(path, 'train.parquet'))
        else:
            train = dt.fread(
                join(path, 'train.csv'),
                columns=['item_id', 'user_id', 'rating', 'timestamp']
            ).to_pandas()
            train['timestamp'] = pd.to_datetime(train['timestamp'])
            train['rating'] = train['rating'].astype('uint8')
            train['user_id'] = train['user_id'].astype('category')
            train['item_id'] = train['item_id'].astype('category')
            self.train = train
            self.train.to_parquet(join(path, 'train.parquet'))
            remove(join(path, 'train.csv'))

    def _save_clean(self, raw):
        clean = join(raw, 'clean')
        mkdir(clean)
        self._fix_movies(raw, clean)
        self._fix_train(raw, clean)
        self._fix_test(raw, clean)

    @staticmethod
    def _fix_test(raw, clean):
        dest = open(join(clean, 'test.csv'), 'w')
        with open(join(raw, 'qualifying.txt')) as source:
            for line in source:
                if line[-2] == ':':
                    movie_id = line[:-2] + ','
                else:
                    dest.write(movie_id + line)
        dest.close()

    def _fix_train(self, raw, clean):
        self.logger.info('Parsing train files')
        folder = join(raw, 'training_set')
        files = glob(join(folder, '*.txt'))
        dest = open(join(clean, 'train.csv'), 'w')
        for file in tqdm(files):
            with open(file) as source:
                for line in source:
                    if line[-2] == ':':
                        movie_id = line[:-2] + ','
                    else:
                        dest.write(movie_id + line)
        dest.close()

    @staticmethod
    def _fix_movies(raw, clean):
        """
        Comma separator also appears in movie titles, for example:
        `72,1974,At Home Among Strangers, A Stranger Among His Own`
        Separator is changed to tabulation for easy parsing.
        """
        file = join(raw, 'movie_titles.txt')
        dest = open(join(clean, 'movies.csv'), 'w')
        with open(file, encoding='ISO-8859-1') as f:
            for line in f.readlines():
                first = line.find(',')
                second = first + 5
                m_id = line[:first]
                year = line[first + 1:second]
                title = line[second + 1:]
                dest.write('\t'.join([m_id, year, title]) + '\n')
        dest.close()

    @safe
    def _download(self, path):
        self.logger.info('Downloading Netflix Prize dataset...')
        url = 'https://archive.org/download/nf_prize_dataset.tar/nf_prize_dataset.tar.gz'
        download_dataset(url, join(self.data_folder, 'netflix.tar.gz'))
        rename(join(self.data_folder, 'download'), path)
        archive = join(path, 'training_set.tar')
        extract(archive)
        rm_if_exists(archive)



from os.path import join, exists

import datatable as dt

from rs_datasets.data_loader import download_dataset
from rs_datasets.generic_dataset import Dataset, safe


class BookCrossing(Dataset):
    def __init__(self, path: str = None):
        """
        :param path: folder which is used to download dataset to
            if it does not contain dataset files.
            If files are found, load them.
        """
        super().__init__(path)
        folder = join(self.data_folder, 'bookx')
        if not exists(folder):
            self._download(folder)

        self.ratings = dt.fread(
            join(folder, 'BX-Book-Ratings.csv'),
            columns=['user_id', 'item_id', 'rating']
        ).to_pandas()

        self.items = dt.fread(
            join(folder, 'BX-Books.csv'),
            columns=['item_id', 'title', 'author', 'year',
                     'publisher', 'img_s', 'img_m', 'img_l']
        ).to_pandas()

        users = dt.fread(
            join(folder, 'BX-Users.csv'),
            columns=['user_id', 'location', 'age']
        ).to_pandas()

        users['age'] = users['age'].replace("NULL", "nan").astype('float')
        self.users = users

    @safe
    def _download(self, path):
        self.logger.info('Downloading Book-Crossing Dataset...')
        url = 'http://www2.informatik.uni-freiburg.de/~cziegler/BX/BX-CSV-Dump.zip'
        download_dataset(url, join(self.data_folder, 'bookx.zip'))



import logging
import os
import shutil
from os.path import join

from pandas import DataFrame, Series
from datatable import Frame


class Dataset:
    def __init__(self, path: str = None):
        data_folder = (path or os.getenv('RS_DATASETS', None) or
                       self.default_folder)
        if not os.path.exists(data_folder):
            os.makedirs(data_folder)
        self.data_folder = data_folder
        logger = logging.getLogger("rs_datasets")
        logger.setLevel(logging.INFO)
        self.logger = logger
        try:
            display = __import__('IPython.core.display', globals(), locals(), ['display'])
            self.display = display.display
        except Exception:
            self.display = print

    @property
    def default_folder(self):
        root = os.path.expanduser('~')
        return join(root, '.rs_datasets')

    def info(self):
        for name, df in self.__dict__.items():
            if isinstance(df, (Frame, DataFrame, Series)):
                print(name)
                self.display(df.head(3))
                print()


def safe(func):
    def decorated(self, path, *args, **kwargs):
        try:
            func(self, path, *args, **kwargs)
        except Exception as e:
            if os.path.exists(path):
                shutil.rmtree(path)
            raise e
    return decorated



from os import rename
from os.path import join, exists

import datatable as dt

from rs_datasets.data_loader import download_dataset
from rs_datasets.generic_dataset import Dataset, safe


class Lastfm(Dataset):
    def __init__(self, path: str = None):
        super().__init__(path)
        folder = join(self.data_folder, 'lastfm')
        if not exists(folder):
            self._download(folder)

        self.play_counts = dt.fread(
            join(folder, 'usersha1-artmbid-artname-plays.tsv'),
            columns=['user_id', 'artist_id', 'artist_name', 'play_count']
        ).to_pandas()

        self.users = dt.fread(
            join(folder, 'usersha1-profile.tsv'),
            columns=['user_id', 'gender', 'age', 'country', 'signup_date']
        ).to_pandas()

    @safe
    def _download(self, path):
        self.logger.info('Downloading Last.fm 360k dataset...')
        url = 'http://mtg.upf.edu/static/datasets/last.fm/lastfm-dataset-360K.tar.gz'
        download_dataset(url, join(self.data_folder, 'lastfm.tar.gz'))
        rename(join(self.data_folder, 'lastfm-dataset-360K'), path)



import os
from os import rename
from os.path import join

import datatable as dt
import pandas as pd

from rs_datasets.data_loader import download_dataset, download_url
from rs_datasets.generic_dataset import Dataset, safe


class MillionSongDataset(Dataset):
    def __init__(
            self,
            merge_kaggle_splits: bool = True,
            drop_mismatches: bool = True,
            path: str = None
    ):
        """
        :param merge_kaggle_splits:
            In MSD Challenge on [Kaggle](https://www.kaggle.com/c/msdchallenge) there were
            public and private parts. By default they are merged together. You can change this, setting
            `merge_kaggle_splits` to `False`.
        :param drop_mismatches:
            There is a [matching error](http://millionsongdataset.com/blog/12-2-12-fixing-matching-errors/)
            between track ids and song ids in MSD. It shouldn't matter if you don't use audio features, but
            by default these items are removed.
        :param path: where to read dataset from or where to download to.
        """
        super().__init__(path)
        folder = join(self.data_folder, 'msd')
        if not os.path.exists(folder):
            self._download(folder)

        try_cache = merge_kaggle_splits and drop_mismatches
        processed = join(folder, 'clean')
        if try_cache and os.path.exists(processed):
            self.train = dt.fread(join(processed, 'train.csv')).to_pandas()
            self.val = dt.fread(join(processed, 'val.csv')).to_pandas()
            self.test = dt.fread(join(processed, 'test.csv')).to_pandas()
        else:
            eval_folder = join(folder, 'evaluation')
            self.train = self._read_triplets(join(folder,
                                                  'train_triplets.txt'))
            val_vis = self._read_triplets(join(eval_folder,
                                               'year1_valid_triplets_visible.txt'))
            val_hid = self._read_triplets(join(eval_folder,
                                               'year1_valid_triplets_hidden.txt'))
            test_vis = self._read_triplets(join(eval_folder,
                                                'year1_test_triplets_visible.txt'))
            test_hid = self._read_triplets(join(eval_folder,
                                                'year1_test_triplets_hidden.txt'))
            if drop_mismatches:
                mismatches = self._read_mismatches(folder)
                mismatches = set(mismatches.item_id)
                self.train = self._drop_mismatches(self.train, mismatches)
                val_vis = self._drop_mismatches(val_vis, mismatches)
                val_hid = self._drop_mismatches(val_hid, mismatches)
                test_vis = self._drop_mismatches(test_vis, mismatches)
                test_hid = self._drop_mismatches(test_hid, mismatches)

            if merge_kaggle_splits:
                self.val = pd.concat([val_vis, val_hid], ignore_index=True)
                self.test = pd.concat([test_vis, test_hid], ignore_index=True)
            else:
                self.val_visible = val_vis
                self.val_hidden = val_hid
                self.test_visible = test_vis
                self.test_hidden = test_hid

            if try_cache and not os.path.exists(processed):
                os.mkdir(processed)
                self.train.to_csv(join(processed, 'train.csv'), index=False)
                self.val.to_csv(join(processed, 'val.csv'), index=False)
                self.test.to_csv(join(processed, 'test.csv'), index=False)

    @staticmethod
    def _read_triplets(path):
        return dt.fread(
            path,
            columns=['user_id', 'item_id', 'play_count']
        ).to_pandas().dropna()

    @staticmethod
    def _read_mismatches(path):
        name = 'sid_mismatches.txt'
        file = join(path, name)
        mismatches = []
        with open(file) as f:
            for line in f.readlines():
                song, track = line[
                    line.find('<') + 1: line.find('>')].split(' ')
                mismatches.append([song, track])
        return pd.DataFrame(mismatches, columns=['item_id', 'track_id'])

    @staticmethod
    def _drop_mismatches(df, mismatches):
        return df[~df.item_id.isin(mismatches)]

    @safe
    def _download(self, path):
        """
        Downloads train triplets, MSD Challenge Kaggle data
        (http://millionsongdataset.com/challenge/)
        and a list of matching errors
        http://millionsongdataset.com/blog/12-2-12-fixing-matching-errors/

        :param path: path to save
        :return: None
        """
        self.logger.info('Getting Million Song Dataset...')
        self.logger.info('Downloading Echo Nest Taste Subprofile train data...')
        base_url = 'http://millionsongdataset.com/sites/default/files/challenge/'

        download_dataset(
            base_url + 'train_triplets.txt.zip',
            join(self.data_folder, 'train.zip')
        )
        rename(join(self.data_folder, 'train'), path)

        self.logger.info('Downloading evaluation data for MSD Challenge...')
        download_dataset(
            base_url + 'EvalDataYear1MSDWebsite.zip',
            join(path, 'eval.zip')
        )
        rename(
            join(path, 'EvalDataYear1MSDWebsite'),
            join(path, 'evaluation')
        )

        self.logger.info('Downloading list of matching errors...')
        url = 'http://millionsongdataset.com/sites/default/files/tasteprofile/sid_mismatches.txt'
        download_url(url, join(path, 'sid_mismatches.txt'))



from os.path import join, exists

import datatable as dt

from rs_datasets.data_loader import download_dataset
from rs_datasets.generic_dataset import Dataset, safe


class YooChoose(Dataset):
    def __init__(self, path: str = None):
        """
        :param path: folder which is used to download dataset to
            if it does not contain dataset files.
            If files are found, load them.
        """
        super().__init__(path)
        folder = join(self.data_folder, 'yoochoose')
        if not exists(folder):
            self._download(folder)

        self.log = dt.fread(
            join(folder, 'yoochoose-clicks.dat'),
            columns=['session_id', 'ts', 'item_id', 'category']
        ).to_pandas()

        self.purchases = dt.fread(
            join(folder, 'yoochoose-buys.dat'),
            columns=['session_id', 'ts', 'item_id', 'price', 'quantity']
        ).to_pandas()

        self.test = dt.fread(
            join(folder, 'yoochoose-test.dat'),
            columns=['session_id', 'ts', 'item_id', 'category']
        ).to_pandas()


    @safe
    def _download(self, path):
        self.logger.info('Downloading YooChoose Dataset...')
        url = 'https://s3-eu-west-1.amazonaws.com/yc-rdata/yoochoose-data.7z'
        download_dataset(url, join(self.data_folder, 'yoochoose.7z'))



import logging
from os.path import join, exists

import datatable as dt

from rs_datasets.generic_dataset import Dataset, safe


class RetailRocket(Dataset):
    def __init__(self, path: str = None):
        super().__init__(path)
        folder = join(self.data_folder, 'retail_rocket')
        if not exists(folder):
            self._download(folder)

        self.category_tree = dt.fread(
            join(folder, 'category_tree.csv'),
            columns=[
                'category_id', 'parent_id'
            ],
        ).to_pandas()

        self.log = dt.fread(
            join(folder, 'events.csv'),
            columns=[
                'ts', 'user_id', 'event', 'item_id', 'transaction_id'
            ],
        ).to_pandas()

        items1 = dt.fread(
            join(folder, 'item_properties_part1.csv'),
            columns=[
                'ts', 'item_id', 'property', 'value'
            ],
        ).to_pandas()

        items2 = dt.fread(
            join(folder, 'item_properties_part2.csv'),
            columns=[
                'ts', 'item_id', 'property', 'value'
            ],
        ).to_pandas()

        self.items = items1.append(items2, ignore_index=True)

    @safe
    def _download(self, path):
        from kaggle.api.kaggle_api_extended import KaggleApi
        logging.info('Downloading Retail Rocket dataset...')
        api = KaggleApi()
        api.authenticate()
        api.dataset_download_files('retailrocket/ecommerce-dataset', path, unzip=True, quiet=False)



import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='rs_datasets',
    version='0.5.1',
    author='Yan-Martin Tamm',
    author_email='darel142857@gmail.com',
    description='Tool for autodownloading recommendation systems datasets',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/Darel13712/rs_datasets',
    packages=setuptools.find_packages(),
    install_requires=[
        'datatable',
        'pandas',
        'gdown',
        'pyarrow',
        'tqdm',
        'xlrd',
        'kaggle',
        'py7zr',
        'openpyxl'
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
)
